@@ -23,14 +23,17 @@ arrow_dplyr_query <- function(.data) {
   # An arrow_dplyr_query is a container for an Arrow data object (Table,
   # RecordBatch, or Dataset) and the state of the user's dplyr query--things
   # like selected columns, filters, and group vars.
+  # An arrow_dplyr_query can contain another arrow_dplyr_query in .data
+  gv <- tryCatch(
+    # If dplyr is not available, or if the input doesn't have a group_vars
+    # method, assume no group vars
+    dplyr::group_vars(.data) %||% character(),
+    error = function(e) character()
+  )
 
-  # For most dplyr methods,
-  # method.Table == method.RecordBatch == method.Dataset == method.arrow_dplyr_query
-  # This works because the functions all pass .data through arrow_dplyr_query()
-  if (inherits(.data, "arrow_dplyr_query")) {
-    return(.data)
+  if (inherits(.data, "data.frame")) {
+    .data <- Table$create(.data)
   }
-
   # Evaluating expressions on a dataset with duplicated fieldnames will error
   dupes <- duplicated(names(.data))
   if (any(dupes)) {
@@ -42,24 +45,19 @@ arrow_dplyr_query <- function(.data) {
       )
     ))
   }
-  
   structure(
     list(
-      .data = if (inherits(.data, "Dataset")) {
-        .data$clone()
-      } else {
-        InMemoryDataset$create(.data)
-      },
+      .data = .data,
       # selected_columns is a named list:
       # * contents are references/expressions pointing to the data
       # * names are the names they should be in the end (i.e. this
       #   records any renaming)
-      selected_columns = make_field_refs(names(.data)),
+      selected_columns = make_field_refs(names(.data$schema)),
       # filtered_rows will be an Expression
       filtered_rows = TRUE,
       # group_by_vars is a character vector of columns (as renamed)
       # in the data. They will be kept when data is pulled into R.
-      group_by_vars = character(),
+      group_by_vars = gv,
       # drop_empty_groups is a logical value indicating whether to drop
       # groups formed by factor levels that don't appear in the data. It
       # should be non-null only when the data is grouped.
@@ -75,6 +73,21 @@ arrow_dplyr_query <- function(.data) {
   )
 }
 
+# The only difference between `arrow_dplyr_query()` and `as_adq()` is that if
+# `.data` is already an `arrow_dplyr_query`, `as_adq()`, will return it as is, but
+# `arrow_dplyr_query()` will nest it inside a new `arrow_dplyr_query`. The only
+# place where `arrow_dplyr_query()` should be called directly is inside
+# `collapse()` methods; everywhere else, call `as_adq()`.
+as_adq <- function(.data) {
+  # For most dplyr methods,
+  # method.Table == method.RecordBatch == method.Dataset == method.arrow_dplyr_query
+  # This works because the functions all pass .data through as_adq()
+  if (inherits(.data, "arrow_dplyr_query")) {
+    return(.data)
+  }
+  arrow_dplyr_query(.data)
+}
+
 make_field_refs <- function(field_names) {
   set_names(lapply(field_names, Expression$field_ref), field_names)
 }
@@ -96,9 +109,14 @@ print.arrow_dplyr_query <- function(x, ...) {
     }
   })
   fields <- paste(names(types), types, sep = ": ", collapse = "\n")
-  cat(class(x$.data)[1], " (query)\n", sep = "")
+  cat(class(source_data(x))[1], " (query)\n", sep = "")
   cat(fields, "\n", sep = "")
   cat("\n")
+  if (length(x$aggregations)) {
+    cat("* Aggregations:\n")
+    aggs <- paste0(names(x$aggregations), ": ", map_chr(x$aggregations, format_aggregation), collapse = "\n")
+    cat(aggs, "\n", sep = "")
+  }
   if (!isTRUE(x$filtered_rows)) {
     filter_string <- x$filtered_rows$ToString()
     cat("* Filter: ", filter_string, "\n", sep = "")
@@ -133,10 +151,20 @@ names.arrow_dplyr_query <- function(x) names(x$selected_columns)
 dim.arrow_dplyr_query <- function(x) {
   cols <- length(names(x))
 
-  if (isTRUE(x$filtered)) {
+  if (is_collapsed(x)) {
+    # Don't evaluate just for nrow
+    rows <- NA_integer_
+  } else if (isTRUE(x$filtered_rows)) {
     rows <- x$.data$num_rows
-  } else {
+  } else if (query_on_dataset(x)) {
+    # TODO: do this with an ExecPlan instead of Scanner (after ARROW-12311)?
+    # See also https://github.com/apache/arrow/pull/12533/files#r818129459
     rows <- Scanner$create(x)$CountRows()
+  } else {
+    # Query on in-memory Table, so evaluate the filter
+    # Don't need any columns
+    x <- select.arrow_dplyr_query(x, NULL)
+    rows <- nrow(compute.arrow_dplyr_query(x))
   }
   c(rows, cols)
 }
@@ -148,19 +176,33 @@ as.data.frame.arrow_dplyr_query <- function(x, row.names = NULL, optional = FALS
 
 #' @export
 head.arrow_dplyr_query <- function(x, n = 6L, ...) {
-  out <- head.Dataset(x, n, ...)
-  restore_dplyr_features(out, x)
+  x$head <- n
+  collapse.arrow_dplyr_query(x)
 }
 
 #' @export
 tail.arrow_dplyr_query <- function(x, n = 6L, ...) {
-  out <- tail.Dataset(x, n, ...)
-  restore_dplyr_features(out, x)
+  x$tail <- n
+  collapse.arrow_dplyr_query(x)
 }
 
 #' @export
-`[.arrow_dplyr_query` <- `[.Dataset`
-# TODO: ^ should also probably restore_dplyr_features, and/or that should be moved down
+`[.arrow_dplyr_query` <- function(x, i, j, ..., drop = FALSE) {
+  x <- ensure_group_vars(x)
+  if (nargs() == 2L) {
+    # List-like column extraction (x[i])
+    return(x[, i])
+  }
+  if (!missing(j)) {
+    x <- select.arrow_dplyr_query(x, all_of(j))
+  }
+
+  if (!missing(i)) {
+    out <- take_dataset_rows(x, i)
+    x <- restore_dplyr_features(out, x)
+  }
+  x
+}
 
 ensure_group_vars <- function(x) {
   if (inherits(x, "arrow_dplyr_query")) {
@@ -191,56 +233,41 @@ ensure_arrange_vars <- function(x) {
   x
 }
 
-restore_dplyr_features <- function(df, query) {
-  # An arrow_dplyr_query holds some attributes that Arrow doesn't know about
-  # After calling collect(), make sure these features are carried over
-
-  if (length(query$group_by_vars) > 0) {
-    # Preserve groupings, if present
-    if (is.data.frame(df)) {
-      df <- dplyr::grouped_df(
-        df,
-        dplyr::group_vars(query),
-        drop = dplyr::group_by_drop_default(query)
-      )
-    } else {
-      # This is a Table, via compute() or collect(as_data_frame = FALSE)
-      df <- arrow_dplyr_query(df)
-      df$group_by_vars <- query$group_by_vars
-      df$drop_empty_groups <- query$drop_empty_groups
-    }
-  }
-  df
-}
-
 # Helper to handle unsupported dplyr features
 # * For Table/RecordBatch, we collect() and then call the dplyr method in R
 # * For Dataset, we just error
-abandon_ship <- function(call, .data, msg = NULL) {
+abandon_ship <- function(call, .data, msg) {
+  msg <- trimws(msg)
   dplyr_fun_name <- sub("^(.*?)\\..*", "\\1", as.character(call[[1]]))
   if (query_on_dataset(.data)) {
-    if (is.null(msg)) {
-      # Default message: function not implemented
-      not_implemented_for_dataset(paste0(dplyr_fun_name, "()"))
-    } else {
-      stop(msg, "\nCall collect() first to pull data into R.", call. = FALSE)
-    }
+    stop(msg, "\nCall collect() first to pull data into R.", call. = FALSE)
   }
   # else, collect and call dplyr method
-  if (!is.null(msg)) {
-    warning(msg, "; pulling data into R", immediate. = TRUE, call. = FALSE)
-  }
+  warning(msg, "; pulling data into R", immediate. = TRUE, call. = FALSE)
   call$.data <- dplyr::collect(.data)
   call[[1]] <- get(dplyr_fun_name, envir = asNamespace("dplyr"))
   eval.parent(call, 2)
 }
 
-query_on_dataset <- function(x) !inherits(x$.data, "InMemoryDataset")
+query_on_dataset <- function(x) inherits(source_data(x), c("Dataset", "RecordBatchReader"))
 
-not_implemented_for_dataset <- function(method) {
-  stop(
-    method, " is not currently implemented for Arrow Datasets. ",
-    "Call collect() first to pull data into R.",
-    call. = FALSE
-  )
+source_data <- function(x) {
+  if (!inherits(x, "arrow_dplyr_query")) {
+    x
+  } else if (is_collapsed(x)) {
+    source_data(x$.data)
+  } else {
+    x$.data
+  }
+}
+
+is_collapsed <- function(x) inherits(x$.data, "arrow_dplyr_query")
+
+has_aggregation <- function(x) {
+  # TODO: update with joins (check right side data too)
+  !is.null(x$aggregations) || (is_collapsed(x) && has_aggregation(x$.data))
+}
+
+has_head_tail <- function(x) {
+  !is.null(x$head) || !is.null(x$tail) || (is_collapsed(x) && has_head_tail(x$.data))
 }
