@@ -118,11 +118,7 @@ Table <- R6Class("Table",
       inherits(other, "Table") && Table__Equals(self, other, isTRUE(check_metadata))
     },
     Validate = function() Table__Validate(self),
-    ValidateFull = function() Table__ValidateFull(self),
-    invalidate = function() {
-      .Call(`_arrow_Table__Reset`, self)
-      super$invalidate()
-    }
+    ValidateFull = function() Table__ValidateFull(self)
   ),
   active = list(
     num_columns = function() Table__num_columns(self),
@@ -153,6 +149,87 @@ Table$create <- function(..., schema = NULL) {
 #' @export
 names.Table <- function(x) x$ColumnNames()
 
+#' Concatenate one or more Tables
+#'
+#' Concatenate one or more [Table] objects into a single table. This operation
+#' does not copy array data, but instead creates new chunked arrays for each
+#' column that point at existing array data.
+#'
+#' @param ... A [Table]
+#' @param unify_schemas If TRUE, the schemas of the tables will be first unified
+#' with fields of the same name being merged, then each table will be promoted
+#' to the unified schema before being concatenated. Otherwise, all tables should
+#' have the same schema.
+#' @examplesIf arrow_available()
+#' tbl <- arrow_table(name = rownames(mtcars), mtcars)
+#' prius <- arrow_table(name = "Prius", mpg = 58, cyl = 4, disp = 1.8)
+#' combined <- concat_tables(tbl, prius)
+#' tail(combined)$to_data_frame()
+#' @export
+concat_tables <- function(..., unify_schemas = TRUE) {
+  tables <- list2(...)
+
+  if (length(tables) == 0) {
+    abort("Must pass at least one Table.")
+  }
+
+  if (!unify_schemas) {
+    # assert they have same schema
+    schema <- tables[[1]]$schema
+    unequal_schema_idx <- which.min(lapply(tables, function(x) x$schema == schema))
+    if (unequal_schema_idx != 1) {
+      abort(c(
+        sprintf("Schema at index %i does not match the first schema.", unequal_schema_idx),
+        i = paste0("Schema 1:\n", schema$ToString()),
+        i = paste0(
+          sprintf("Schema %d:\n", unequal_schema_idx),
+          tables[[unequal_schema_idx]]$schema$ToString()
+        )
+      ))
+    }
+  }
+
+  Table__ConcatenateTables(tables, unify_schemas)
+}
+
+#' @export
+rbind.Table <- function(...) {
+  concat_tables(..., unify_schemas = FALSE)
+}
+
+#' @export
+cbind.Table <- function(...) {
+  call <- sys.call()
+  inputs <- list(...)
+  arg_names <- if (is.null(names(inputs))) {
+    rep("", length(inputs))
+  } else {
+    names(inputs)
+  }
+
+  cbind_check_length(inputs, call)
+
+  columns <- flatten(map(seq_along(inputs), function(i) {
+    input <- inputs[[i]]
+    name <- arg_names[i]
+
+    if (inherits(input, "ArrowTabular")) {
+      set_names(input$columns, names(input))
+    } else if (inherits(input, "data.frame")) {
+      as.list(input)
+    } else {
+      if (name == "") {
+        abort("Vector and array arguments must have names",
+          i = sprintf("Argument ..%d is missing a name", i)
+        )
+      }
+      list2("{name}" := input)
+    }
+  }))
+
+  Table$create(!!!columns)
+}
+
 #' @param ... A `data.frame` or a named set of Arrays or vectors. If given a
 #' mixture of data.frames and named vectors, the inputs will be autospliced together
 #' (see examples). Alternatively, you can provide a single Arrow IPC
@@ -160,7 +237,7 @@ names.Table <- function(x) x$ColumnNames()
 #' @param schema a [Schema], or `NULL` (the default) to infer the schema from
 #' the data in `...`. When providing an Arrow IPC buffer, `schema` is required.
 #' @rdname Table
-#' @examplesIf arrow_available()
+#' @examples
 #' tbl <- arrow_table(name = rownames(mtcars), mtcars)
 #' dim(tbl)
 #' dim(head(tbl))
@@ -170,3 +247,67 @@ names.Table <- function(x) x$ColumnNames()
 #' as.data.frame(tbl[4:8, c("gear", "hp", "wt")])
 #' @export
 arrow_table <- Table$create
+
+
+#' Convert an object to an Arrow Table
+#'
+#' Whereas [arrow_table()] constructs a table from one or more columns,
+#' `as_arrow_table()` converts a single object to an Arrow [Table].
+#'
+#' @param x An object to convert to an Arrow Table
+#' @param ... Passed to S3 methods
+#' @inheritParams arrow_table
+#'
+#' @return A [Table]
+#' @export
+#'
+#' @examples
+#' # use as_arrow_table() for a single object
+#' as_arrow_table(data.frame(col1 = 1, col2 = "two"))
+#'
+#' # use arrow_table() to create from columns
+#' arrow_table(col1 = 1, col2 = "two")
+#'
+as_arrow_table <- function(x, ..., schema = NULL) {
+  UseMethod("as_arrow_table")
+}
+
+#' @rdname as_arrow_table
+#' @export
+as_arrow_table.default <- function(x, ...) {
+  # throw a classed error here so that we can customize the error message
+  # in as_writable_table()
+  abort(
+    sprintf(
+      "No method for `as_arrow_table()` for object of class %s",
+      paste(class(x), collapse = " / ")
+    ),
+    class = "arrow_no_method_as_arrow_table"
+  )
+}
+
+#' @rdname as_arrow_table
+#' @export
+as_arrow_table.Table <- function(x, ..., schema = NULL) {
+  if (is.null(schema)) {
+    x
+  } else {
+    x$cast(schema)
+  }
+}
+
+#' @rdname as_arrow_table
+#' @export
+as_arrow_table.RecordBatch <- function(x, ..., schema = NULL) {
+  if (is.null(schema)) {
+    Table$create(x)
+  } else {
+    Table$create(x$cast(schema))
+  }
+}
+
+#' @rdname as_arrow_table
+#' @export
+as_arrow_table.data.frame <- function(x, ..., schema = NULL) {
+  Table$create(x, schema = schema)
+}
