@@ -391,6 +391,26 @@ std::shared_ptr<ds::PartitioningFactory> dataset___HivePartitioning__MakeFactory
   return ds::HivePartitioning::MakeFactory(options);
 }
 
+// [[dataset::export]]
+std::shared_ptr<arrow::Schema> dataset___PartitioningFactory__Inspect(
+    const std::shared_ptr<ds::PartitioningFactory>& factory,
+    const std::vector<std::string>& paths) {
+  return ValueOrStop(factory->Inspect(paths));
+}
+
+// [[dataset::export]]
+std::shared_ptr<ds::Partitioning> dataset___PartitioningFactory__Finish(
+    const std::shared_ptr<ds::PartitioningFactory>& factory,
+    const std::shared_ptr<arrow::Schema>& schema) {
+  return ValueOrStop(factory->Finish(schema));
+}
+
+// [[dataset::export]]
+std::string dataset___PartitioningFactory__type_name(
+    const std::shared_ptr<ds::PartitioningFactory>& factory) {
+  return factory->type_name();
+}
+
 // ScannerBuilder, Scanner
 
 // [[dataset::export]]
@@ -424,12 +444,6 @@ void dataset___ScannerBuilder__UseThreads(const std::shared_ptr<ds::ScannerBuild
   StopIfNotOk(sb->UseThreads(threads));
 }
 
-// [[dataset::export]]
-void dataset___ScannerBuilder__UseAsync(const std::shared_ptr<ds::ScannerBuilder>& sb,
-                                        bool use_async) {
-  StopIfNotOk(sb->UseAsync(use_async));
-}
-
 // [[dataset::export]]
 void dataset___ScannerBuilder__BatchSize(const std::shared_ptr<ds::ScannerBuilder>& sb,
                                          int64_t batch_size) {
@@ -497,36 +511,6 @@ std::shared_ptr<arrow::Schema> dataset___Scanner__schema(
   return sc->options()->projected_schema;
 }
 
-// [[dataset::export]]
-cpp11::list dataset___ScanTask__get_batches(
-    const std::shared_ptr<ds::ScanTask>& scan_task) {
-  arrow::RecordBatchIterator rbi;
-  rbi = ValueOrStop(scan_task->Execute());
-  std::vector<std::shared_ptr<arrow::RecordBatch>> out;
-  std::shared_ptr<arrow::RecordBatch> batch;
-  for (auto b : rbi) {
-    batch = ValueOrStop(b);
-    out.push_back(batch);
-  }
-  return arrow::r::to_r_list(out);
-}
-
-// [[dataset::export]]
-void dataset___Dataset__Write(
-    const std::shared_ptr<ds::FileWriteOptions>& file_write_options,
-    const std::shared_ptr<fs::FileSystem>& filesystem, std::string base_dir,
-    const std::shared_ptr<ds::Partitioning>& partitioning, std::string basename_template,
-    const std::shared_ptr<ds::Scanner>& scanner, int8_t existing_data_behavior) {
-  ds::FileSystemDatasetWriteOptions opts;
-  opts.file_write_options = file_write_options;
-  opts.existing_data_behavior = (ds::ExistingDataBehavior)existing_data_behavior;
-  opts.filesystem = filesystem;
-  opts.base_dir = base_dir;
-  opts.partitioning = partitioning;
-  opts.basename_template = basename_template;
-  StopIfNotOk(ds::FileSystemDataset::Write(opts, scanner));
-}
-
 // [[dataset::export]]
 std::shared_ptr<arrow::Table> dataset___Scanner__TakeRows(
     const std::shared_ptr<ds::Scanner>& scanner,
