@@ -35,6 +35,52 @@ std::shared_ptr<arrow::RecordBatch> RecordBatchReader__ReadNext(
   return batch;
 }
 
+// [[arrow::export]]
+cpp11::list RecordBatchReader__batches(
+    const std::shared_ptr<arrow::RecordBatchReader>& reader) {
+  return arrow::r::to_r_list(ValueOrStop(reader->ToRecordBatches()));
+}
+
+// [[arrow::export]]
+std::shared_ptr<arrow::RecordBatchReader> RecordBatchReader__from_batches(
+    const std::vector<std::shared_ptr<arrow::RecordBatch>>& batches,
+    cpp11::sexp schema_sxp) {
+  bool infer_schema = !Rf_inherits(schema_sxp, "Schema");
+
+  if (infer_schema) {
+    return ValueOrStop(arrow::RecordBatchReader::Make(std::move(batches)));
+  } else {
+    auto schema = cpp11::as_cpp<std::shared_ptr<arrow::Schema>>(schema_sxp);
+    return ValueOrStop(arrow::RecordBatchReader::Make(std::move(batches), schema));
+  }
+}
+
+// [[arrow::export]]
+std::shared_ptr<arrow::RecordBatchReader> RecordBatchReader__from_Table(
+    const std::shared_ptr<arrow::Table>& table) {
+  return std::make_shared<arrow::TableBatchReader>(table);
+}
+
+// [[arrow::export]]
+std::shared_ptr<arrow::Table> Table__from_RecordBatchReader(
+    const std::shared_ptr<arrow::RecordBatchReader>& reader) {
+  return ValueOrStop(reader->ToTable());
+}
+
+// [[arrow::export]]
+std::shared_ptr<arrow::Table> RecordBatchReader__Head(
+    const std::shared_ptr<arrow::RecordBatchReader>& reader, int64_t num_rows) {
+  std::vector<std::shared_ptr<arrow::RecordBatch>> batches;
+  std::shared_ptr<arrow::RecordBatch> this_batch;
+  while (num_rows > 0) {
+    this_batch = ValueOrStop(reader->Next());
+    if (this_batch == nullptr) break;
+    batches.push_back(this_batch->Slice(0, num_rows));
+    num_rows -= this_batch->num_rows();
+  }
+  return ValueOrStop(arrow::Table::FromRecordBatches(reader->schema(), batches));
+}
+
 // -------- RecordBatchStreamReader
 
 // [[arrow::export]]
@@ -45,22 +91,6 @@ std::shared_ptr<arrow::ipc::RecordBatchStreamReader> ipc___RecordBatchStreamRead
   return ValueOrStop(arrow::ipc::RecordBatchStreamReader::Open(stream, options));
 }
 
-// [[arrow::export]]
-cpp11::list ipc___RecordBatchStreamReader__batches(
-    const std::shared_ptr<arrow::ipc::RecordBatchStreamReader>& reader) {
-  std::vector<std::shared_ptr<arrow::RecordBatch>> res;
-
-  while (true) {
-    std::shared_ptr<arrow::RecordBatch> batch;
-    StopIfNotOk(reader->ReadNext(&batch));
-    if (!batch) break;
-
-    res.push_back(batch);
-  }
-
-  return arrow::r::to_r_list(res);
-}
-
 // -------- RecordBatchFileReader
 
 // [[arrow::export]]
@@ -92,14 +122,6 @@ std::shared_ptr<arrow::ipc::RecordBatchFileReader> ipc___RecordBatchFileReader__
   return ValueOrStop(arrow::ipc::RecordBatchFileReader::Open(file, options));
 }
 
-// [[arrow::export]]
-std::shared_ptr<arrow::Table> Table__from_RecordBatchReader(
-    const std::shared_ptr<arrow::RecordBatchReader>& reader) {
-  std::shared_ptr<arrow::Table> table = nullptr;
-  StopIfNotOk(reader->ReadAll(&table));
-  return table;
-}
-
 // [[arrow::export]]
 std::shared_ptr<arrow::Table> Table__from_RecordBatchFileReader(
     const std::shared_ptr<arrow::ipc::RecordBatchFileReader>& reader) {
