@@ -21,7 +21,15 @@ build_features <- c(
   uncompressed = TRUE
 )
 
+force_tests <- function() {
+  identical(tolower(Sys.getenv("ARROW_R_force_tests()")), "true")
+}
+
 skip_if_not_available <- function(feature) {
+  if (force_tests()) {
+    return()
+  }
+
   if (feature == "re2") {
     # RE2 does not support valgrind (on purpose): https://github.com/google/re2/issues/177
     skip_on_linux_devel()
@@ -37,6 +45,10 @@ skip_if_not_available <- function(feature) {
 }
 
 skip_if_no_pyarrow <- function() {
+  if (force_tests()) {
+    return()
+  }
+
   skip_on_linux_devel()
   skip_on_os("windows")
 
@@ -47,6 +59,10 @@ skip_if_no_pyarrow <- function() {
 }
 
 skip_if_not_dev_mode <- function() {
+  if (force_tests()) {
+    return()
+  }
+
   skip_if_not(
     identical(tolower(Sys.getenv("ARROW_R_DEV")), "true"),
     "environment variable ARROW_R_DEV"
@@ -54,6 +70,10 @@ skip_if_not_dev_mode <- function() {
 }
 
 skip_if_not_running_large_memory_tests <- function() {
+  if (force_tests()) {
+    return()
+  }
+
   skip_if_not(
     identical(tolower(Sys.getenv("ARROW_LARGE_MEMORY_TESTS")), "true"),
     "environment variable ARROW_LARGE_MEMORY_TESTS"
@@ -61,6 +81,10 @@ skip_if_not_running_large_memory_tests <- function() {
 }
 
 skip_on_linux_devel <- function() {
+  if (force_tests()) {
+    return()
+  }
+
   # Skip when the OS is linux + and the R version is development
   # helpful for skipping on Valgrind, and the sanitizer checks (clang + gcc) on cran
   if (on_linux_dev()) {
@@ -69,12 +93,22 @@ skip_on_linux_devel <- function() {
 }
 
 skip_if_r_version <- function(r_version) {
+  if (force_tests()) {
+    return()
+  }
+
   if (getRversion() <= r_version) {
     skip(paste("R version:", getRversion()))
   }
 }
 
 process_is_running <- function(x) {
+  if (force_tests()) {
+    # Return TRUE as this is used as a condition in an if statement
+    # so the behavior is inverted compared to the skip_* functions.
+    return(TRUE)
+  }
+
   cmd <- sprintf("ps aux | grep '%s' | grep -v grep", x)
   tryCatch(system(cmd, ignore.stdout = TRUE) == 0, error = function(e) FALSE)
 }
