@@ -15,7 +15,7 @@
 # specific language governing permissions and limitations
 # under the License.
 
-skip_if_not_available("dataset")
+skip_if(on_old_windows())
 skip_if_not_available("utf8proc")
 
 library(dplyr, warn.conflicts = FALSE)
@@ -35,84 +35,84 @@ test_that("paste, paste0, and str_c", {
   y <- Expression$field_ref("y")
 
   # no NAs in data
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(paste(v, w)) %>%
       collect(),
     df
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(paste(v, w, sep = "-")) %>%
       collect(),
     df
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(paste0(v, w)) %>%
       collect(),
     df
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(str_c(v, w)) %>%
       collect(),
     df
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(str_c(v, w, sep = "+")) %>%
       collect(),
     df
   )
 
   # NAs in data
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(paste(x, y)) %>%
       collect(),
     df
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(paste(x, y, sep = "-")) %>%
       collect(),
     df
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(str_c(x, y)) %>%
       collect(),
     df
   )
 
   # non-character column in dots
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(paste0(x, y, z)) %>%
       collect(),
     df
   )
 
   # literal string in dots
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(paste(x, "foo", y)) %>%
       collect(),
     df
   )
 
   # literal NA in dots
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(paste(x, NA, y)) %>%
       collect(),
     df
   )
 
   # expressions in dots
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(paste0(x, toupper(y), as.character(z))) %>%
       collect(),
     df
@@ -121,20 +121,20 @@ test_that("paste, paste0, and str_c", {
   # sep is literal NA
   # errors in paste() (consistent with base::paste())
   expect_error(
-    nse_funcs$paste(x, y, sep = NA_character_),
+    call_binding("paste", x, y, sep = NA_character_),
     "Invalid separator"
   )
   # emits null in str_c() (consistent with stringr::str_c())
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(str_c(x, y, sep = NA_character_)) %>%
       collect(),
     df
   )
 
   # sep passed in dots to paste0 (which doesn't take a sep argument)
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(paste0(x, y, sep = "-")) %>%
       collect(),
     df
@@ -156,49 +156,55 @@ test_that("paste, paste0, and str_c", {
 
   # collapse argument not supported
   expect_error(
-    nse_funcs$paste(x, y, collapse = ""),
+    call_binding("paste", x, y, collapse = ""),
     "collapse"
   )
   expect_error(
-    nse_funcs$paste0(x, y, collapse = ""),
+    call_binding("paste0", x, y, collapse = ""),
     "collapse"
   )
   expect_error(
-    nse_funcs$str_c(x, y, collapse = ""),
+    call_binding("str_c", x, y, collapse = ""),
     "collapse"
   )
 
   # literal vectors of length != 1 not supported
   expect_error(
-    nse_funcs$paste(x, character(0), y),
+    call_binding("paste", x, character(0), y),
     "Literal vectors of length != 1 not supported in string concatenation"
   )
   expect_error(
-    nse_funcs$paste(x, c(",", ";"), y),
+    call_binding("paste", x, c(",", ";"), y),
     "Literal vectors of length != 1 not supported in string concatenation"
   )
 })
 
 test_that("grepl with ignore.case = FALSE and fixed = TRUE", {
-  df <- tibble(x = c("Foo", "bar"))
-  expect_dplyr_equal(
-    input %>%
+  df <- tibble(x = c("Foo", "bar", NA_character_))
+  compare_dplyr_binding(
+    .input %>%
       filter(grepl("o", x, fixed = TRUE)) %>%
       collect(),
     df
   )
+  compare_dplyr_binding(
+    .input %>%
+      mutate(x = grepl("o", x, fixed = TRUE)) %>%
+      collect(),
+    df
+  )
 })
 
 test_that("sub and gsub with ignore.case = FALSE and fixed = TRUE", {
   df <- tibble(x = c("Foo", "bar"))
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(x = sub("Foo", "baz", x, fixed = TRUE)) %>%
       collect(),
     df
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(x = gsub("o", "u", x, fixed = TRUE)) %>%
       collect(),
     df
@@ -209,23 +215,23 @@ test_that("sub and gsub with ignore.case = FALSE and fixed = TRUE", {
 skip_if_not_available("re2")
 
 test_that("grepl", {
-  df <- tibble(x = c("Foo", "bar"))
+  df <- tibble(x = c("Foo", "bar", NA_character_))
 
   for (fixed in c(TRUE, FALSE)) {
-    expect_dplyr_equal(
-      input %>%
+    compare_dplyr_binding(
+      .input %>%
         filter(grepl("Foo", x, fixed = fixed)) %>%
         collect(),
       df
     )
-    expect_dplyr_equal(
-      input %>%
+    compare_dplyr_binding(
+      .input %>%
         transmute(x = grepl("^B.+", x, ignore.case = FALSE, fixed = fixed)) %>%
         collect(),
       df
     )
-    expect_dplyr_equal(
-      input %>%
+    compare_dplyr_binding(
+      .input %>%
         filter(grepl("Foo", x, ignore.case = FALSE, fixed = fixed)) %>%
         collect(),
       df
@@ -234,10 +240,10 @@ test_that("grepl", {
 })
 
 test_that("grepl with ignore.case = TRUE and fixed = TRUE", {
-  df <- tibble(x = c("Foo", "bar"))
+  df <- tibble(x = c("Foo", "bar", NA_character_))
 
   # base::grepl() ignores ignore.case = TRUE with a warning when fixed = TRUE,
-  # so we can't use expect_dplyr_equal() for these tests
+  # so we can't use compare_dplyr_binding() for these tests
   expect_equal(
     df %>%
       Table$create() %>%
@@ -248,53 +254,65 @@ test_that("grepl with ignore.case = TRUE and fixed = TRUE", {
   expect_equal(
     df %>%
       Table$create() %>%
-      filter(x = grepl("^B.+", x, ignore.case = TRUE, fixed = TRUE)) %>%
+      filter(grepl("^B.+", x, ignore.case = TRUE, fixed = TRUE)) %>%
       collect(),
     tibble(x = character(0))
   )
+  expect_equal(
+    df %>%
+      Table$create() %>%
+      mutate(
+        a = grepl("O", x, ignore.case = TRUE, fixed = TRUE)
+      ) %>%
+      collect(),
+    tibble(
+      x = c("Foo", "bar", NA_character_),
+      a = c(TRUE, FALSE, FALSE)
+    )
+  )
 })
 
 test_that("str_detect", {
-  df <- tibble(x = c("Foo", "bar"))
+  df <- tibble(x = c("Foo", "bar", NA_character_))
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       filter(str_detect(x, regex("^F"))) %>%
       collect(),
     df
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(x = str_detect(x, regex("^f[A-Z]{2}", ignore_case = TRUE))) %>%
       collect(),
     df
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(x = str_detect(x, regex("^f[A-Z]{2}", ignore_case = TRUE), negate = TRUE)) %>%
       collect(),
     df
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       filter(str_detect(x, fixed("o"))) %>%
       collect(),
     df
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       filter(str_detect(x, fixed("O"))) %>%
       collect(),
     df
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       filter(str_detect(x, fixed("O", ignore_case = TRUE))) %>%
       collect(),
     df
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       filter(str_detect(x, fixed("O", ignore_case = TRUE), negate = TRUE)) %>%
       collect(),
     df
@@ -305,20 +323,20 @@ test_that("sub and gsub", {
   df <- tibble(x = c("Foo", "bar"))
 
   for (fixed in c(TRUE, FALSE)) {
-    expect_dplyr_equal(
-      input %>%
+    compare_dplyr_binding(
+      .input %>%
         transmute(x = sub("Foo", "baz", x, fixed = fixed)) %>%
         collect(),
       df
     )
-    expect_dplyr_equal(
-      input %>%
+    compare_dplyr_binding(
+      .input %>%
         transmute(x = sub("^B.+", "baz", x, ignore.case = FALSE, fixed = fixed)) %>%
         collect(),
       df
     )
-    expect_dplyr_equal(
-      input %>%
+    compare_dplyr_binding(
+      .input %>%
         transmute(x = sub("Foo", "baz", x, ignore.case = FALSE, fixed = fixed)) %>%
         collect(),
       df
@@ -330,7 +348,7 @@ test_that("sub and gsub with ignore.case = TRUE and fixed = TRUE", {
   df <- tibble(x = c("Foo", "bar"))
 
   # base::sub() and base::gsub() ignore ignore.case = TRUE with a warning when
-  # fixed = TRUE, so we can't use expect_dplyr_equal() for these tests
+  # fixed = TRUE, so we can't use compare_dplyr_binding() for these tests
   expect_equal(
     df %>%
       Table$create() %>%
@@ -357,47 +375,47 @@ test_that("sub and gsub with ignore.case = TRUE and fixed = TRUE", {
 test_that("str_replace and str_replace_all", {
   df <- tibble(x = c("Foo", "bar"))
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(x = str_replace_all(x, "^F", "baz")) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(x = str_replace_all(x, regex("^F"), "baz")) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(x = str_replace(x, "^F[a-z]{2}", "baz")) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(x = str_replace(x, regex("^f[A-Z]{2}", ignore_case = TRUE), "baz")) %>%
       collect(),
     df
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(x = str_replace_all(x, fixed("o"), "u")) %>%
       collect(),
     df
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(x = str_replace(x, fixed("O"), "u")) %>%
       collect(),
     df
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(x = str_replace(x, fixed("O", ignore_case = TRUE), "u")) %>%
       collect(),
     df
@@ -407,8 +425,8 @@ test_that("str_replace and str_replace_all", {
 test_that("strsplit and str_split", {
   df <- tibble(x = c("Foo and bar", "baz and qux and quux"))
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(x = strsplit(x, "and")) %>%
       collect(),
     df,
@@ -416,50 +434,50 @@ test_that("strsplit and str_split", {
     # has type information in it, but it's just a bare list from R/dplyr.
     ignore_attr = TRUE
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(x = strsplit(x, "and.*", fixed = TRUE)) %>%
       collect(),
     df,
     ignore_attr = TRUE
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(x = strsplit(x, " +and +")) %>%
       collect(),
     df,
     ignore_attr = TRUE
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(x = str_split(x, "and")) %>%
       collect(),
     df,
     ignore_attr = TRUE
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(x = str_split(x, "and", n = 2)) %>%
       collect(),
     df,
     ignore_attr = TRUE
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(x = str_split(x, fixed("and"), n = 2)) %>%
       collect(),
     df,
     ignore_attr = TRUE
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(x = str_split(x, regex("and"), n = 2)) %>%
       collect(),
     df,
     ignore_attr = TRUE
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(x = str_split(x, "Foo|bar", n = 2)) %>%
       collect(),
     df,
@@ -467,10 +485,29 @@ test_that("strsplit and str_split", {
   )
 })
 
+test_that("strrep and str_dup", {
+  df <- tibble(x = c("foo1", " \tB a R\n", "!apACHe aRroW!"))
+  for (times in 0:8) {
+    compare_dplyr_binding(
+      .input %>%
+        mutate(x = strrep(x, times)) %>%
+        collect(),
+      df
+    )
+
+    compare_dplyr_binding(
+      .input %>%
+        mutate(x = str_dup(x, times)) %>%
+        collect(),
+      df
+    )
+  }
+})
+
 test_that("str_to_lower, str_to_upper, and str_to_title", {
   df <- tibble(x = c("foo1", " \tB a R\n", "!apACHe aRroW!"))
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(
         x_lower = str_to_lower(x),
         x_upper = str_to_upper(x),
@@ -482,8 +519,8 @@ test_that("str_to_lower, str_to_upper, and str_to_title", {
 
   # Error checking a single function because they all use the same code path.
   expect_error(
-    nse_funcs$str_to_lower("Apache Arrow", locale = "sp"),
-    "Providing a value for 'locale' other than the default ('en') is not supported by Arrow",
+    call_binding("str_to_lower", "Apache Arrow", locale = "sp"),
+    "Providing a value for 'locale' other than the default ('en') is not supported in Arrow",
     fixed = TRUE
   )
 })
@@ -546,27 +583,27 @@ test_that("errors and warnings in string splitting", {
 
   x <- Expression$field_ref("x")
   expect_error(
-    nse_funcs$str_split(x, fixed("and", ignore_case = TRUE)),
-    "Case-insensitive string splitting not supported by Arrow"
+    call_binding("str_split", x, fixed("and", ignore_case = TRUE)),
+    "Case-insensitive string splitting not supported in Arrow"
   )
   expect_error(
-    nse_funcs$str_split(x, coll("and.?")),
-    "Pattern modifier `coll()` not supported by Arrow",
+    call_binding("str_split", x, coll("and.?")),
+    "Pattern modifier `coll()` not supported in Arrow",
     fixed = TRUE
   )
   expect_error(
-    nse_funcs$str_split(x, boundary(type = "word")),
-    "Pattern modifier `boundary()` not supported by Arrow",
+    call_binding("str_split", x, boundary(type = "word")),
+    "Pattern modifier `boundary()` not supported in Arrow",
     fixed = TRUE
   )
   expect_error(
-    nse_funcs$str_split(x, "and", n = 0),
-    "Splitting strings into zero parts not supported by Arrow"
+    call_binding("str_split", x, "and", n = 0),
+    "Splitting strings into zero parts not supported in Arrow"
   )
 
   # This condition generates a warning
   expect_warning(
-    nse_funcs$str_split(x, fixed("and"), simplify = TRUE),
+    call_binding("str_split", x, fixed("and"), simplify = TRUE),
     "Argument 'simplify = TRUE' will be ignored"
   )
 })
@@ -575,19 +612,19 @@ test_that("errors and warnings in string detection and replacement", {
   x <- Expression$field_ref("x")
 
   expect_error(
-    nse_funcs$str_detect(x, boundary(type = "character")),
-    "Pattern modifier `boundary()` not supported by Arrow",
+    call_binding("str_detect", x, boundary(type = "character")),
+    "Pattern modifier `boundary()` not supported in Arrow",
     fixed = TRUE
   )
   expect_error(
-    nse_funcs$str_replace_all(x, coll("o", locale = "en"), "ó"),
-    "Pattern modifier `coll()` not supported by Arrow",
+    call_binding("str_replace_all", x, coll("o", locale = "en"), "ó"),
+    "Pattern modifier `coll()` not supported in Arrow",
     fixed = TRUE
   )
 
   # This condition generates a warning
   expect_warning(
-    nse_funcs$str_replace_all(x, regex("o", multiline = TRUE), "u"),
+    call_binding("str_replace_all", x, regex("o", multiline = TRUE), "u"),
     "Ignoring pattern modifier argument not supported in Arrow: \"multiline\""
   )
 })
@@ -596,8 +633,8 @@ test_that("backreferences in pattern in string detection", {
   skip("RE2 does not support backreferences in pattern (https://github.com/google/re2/issues/101)")
   df <- tibble(x = c("Foo", "bar"))
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       filter(str_detect(x, regex("F([aeiou])\\1"))) %>%
       collect(),
     df
@@ -607,8 +644,8 @@ test_that("backreferences in pattern in string detection", {
 test_that("backreferences (substitutions) in string replacement", {
   df <- tibble(x = c("Foo", "bar"))
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(desc = sub(
         "(?:https?|ftp)://([^/\r\n]+)(/[^\r\n]*)?",
         "path `\\2` on server `\\1`",
@@ -617,20 +654,20 @@ test_that("backreferences (substitutions) in string replacement", {
       collect(),
     tibble(url = "https://arrow.apache.org/docs/r/")
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(x = str_replace(x, "^(\\w)o(.*)", "\\1\\2p")) %>%
       collect(),
     df
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(x = str_replace(x, regex("^(\\w)o(.*)", ignore_case = TRUE), "\\1\\2p")) %>%
       collect(),
     df
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(x = str_replace(x, regex("^(\\w)o(.*)", ignore_case = TRUE), "\\1\\2p")) %>%
       collect(),
     df
@@ -640,7 +677,7 @@ test_that("backreferences (substitutions) in string replacement", {
 test_that("edge cases in string detection and replacement", {
   # in case-insensitive fixed match/replace, test that "\\E" in the search
   # string and backslashes in the replacement string are interpreted literally.
-  # this test does not use expect_dplyr_equal() because base::sub() and
+  # this test does not use compare_dplyr_binding() because base::sub() and
   # base::grepl() do not support ignore.case = TRUE when fixed = TRUE.
   expect_equal(
     tibble(x = c("\\Q\\e\\D")) %>%
@@ -659,245 +696,20 @@ test_that("edge cases in string detection and replacement", {
 
   # test that a user's "(?i)" prefix does not break the "(?i)" prefix that's
   # added in case-insensitive regex match/replace
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       filter(grepl("(?i)^[abc]{3}$", x, ignore.case = TRUE, fixed = FALSE)) %>%
       collect(),
     tibble(x = c("ABC"))
   )
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       transmute(x = sub("(?i)^[abc]{3}$", "123", x, ignore.case = TRUE, fixed = FALSE)) %>%
       collect(),
     tibble(x = c("ABC"))
   )
 })
 
-test_that("strptime", {
-  # base::strptime() defaults to local timezone
-  # but arrow's strptime defaults to UTC.
-  # So that tests are consistent, set the local timezone to UTC
-  # TODO: consider reevaluating this workaround after ARROW-12980
-  withr::local_timezone("UTC")
-
-  t_string <- tibble(x = c("2018-10-07 19:04:05", NA))
-  t_stamp <- tibble(x = c(lubridate::ymd_hms("2018-10-07 19:04:05"), NA))
-
-  expect_equal(
-    t_string %>%
-      Table$create() %>%
-      mutate(
-        x = strptime(x)
-      ) %>%
-      collect(),
-    t_stamp,
-    ignore_attr = "tzone"
-  )
-
-  expect_equal(
-    t_string %>%
-      Table$create() %>%
-      mutate(
-        x = strptime(x, format = "%Y-%m-%d %H:%M:%S")
-      ) %>%
-      collect(),
-    t_stamp,
-    ignore_attr = "tzone"
-  )
-
-  expect_equal(
-    t_string %>%
-      Table$create() %>%
-      mutate(
-        x = strptime(x, format = "%Y-%m-%d %H:%M:%S", unit = "ns")
-      ) %>%
-      collect(),
-    t_stamp,
-    ignore_attr = "tzone"
-  )
-
-  expect_equal(
-    t_string %>%
-      Table$create() %>%
-      mutate(
-        x = strptime(x, format = "%Y-%m-%d %H:%M:%S", unit = "s")
-      ) %>%
-      collect(),
-    t_stamp,
-    ignore_attr = "tzone"
-  )
-
-  tstring <- tibble(x = c("08-05-2008", NA))
-  tstamp <- strptime(c("08-05-2008", NA), format = "%m-%d-%Y")
-
-  expect_equal(
-    tstring %>%
-      Table$create() %>%
-      mutate(
-        x = strptime(x, format = "%m-%d-%Y")
-      ) %>%
-      pull(),
-    # R's strptime returns POSIXlt (list type)
-    as.POSIXct(tstamp),
-    ignore_attr = "tzone"
-  )
-})
-
-test_that("errors in strptime", {
-  # Error when tz is passed
-  x <- Expression$field_ref("x")
-  expect_error(
-    nse_funcs$strptime(x, tz = "PDT"),
-    "Time zone argument not supported by Arrow"
-  )
-})
-
-test_that("strftime", {
-  skip_on_os("windows") # https://issues.apache.org/jira/browse/ARROW-13168
-
-  times <- tibble(
-    datetime = c(lubridate::ymd_hms("2018-10-07 19:04:05", tz = "Etc/GMT+6"), NA),
-    date = c(as.Date("2021-01-01"), NA)
-  )
-  formats <- "%a %A %w %d %b %B %m %y %Y %H %I %p %M %z %Z %j %U %W %x %X %% %G %V %u"
-  formats_date <- "%a %A %w %d %b %B %m %y %Y %H %I %p %M %j %U %W %x %X %% %G %V %u"
-
-  expect_dplyr_equal(
-    input %>%
-      mutate(x = strftime(datetime, format = formats)) %>%
-      collect(),
-    times
-  )
-
-  expect_dplyr_equal(
-    input %>%
-      mutate(x = strftime(date, format = formats_date)) %>%
-      collect(),
-    times
-  )
-
-  expect_dplyr_equal(
-    input %>%
-      mutate(x = strftime(datetime, format = formats, tz = "Pacific/Marquesas")) %>%
-      collect(),
-    times
-  )
-
-  expect_dplyr_equal(
-    input %>%
-      mutate(x = strftime(datetime, format = formats, tz = "EST", usetz = TRUE)) %>%
-      collect(),
-    times
-  )
-
-  withr::with_timezone(
-    "Pacific/Marquesas", {
-      expect_dplyr_equal(
-        input %>%
-          mutate(
-            x = strftime(datetime, format = formats, tz = "EST"),
-            x_date = strftime(date, format = formats_date, tz = "EST")
-          ) %>%
-          collect(),
-        times
-      )
-
-      expect_dplyr_equal(
-        input %>%
-          mutate(
-            x = strftime(datetime, format = formats),
-            x_date = strftime(date, format = formats_date)
-          ) %>%
-          collect(),
-        times
-      )
-    }
-  )
-
-  # This check is due to differences in the way %c currently works in Arrow and R's strftime.
-  # We can revisit after https://github.com/HowardHinnant/date/issues/704 is resolved.
-  expect_error(
-    times %>%
-      Table$create() %>%
-      mutate(x = strftime(datetime, format = "%c")) %>%
-      collect(),
-    "%c flag is not supported in non-C locales."
-  )
-
-  # Output precision of %S depends on the input timestamp precision.
-  # Timestamps with second precision are represented as integers while
-  # milliseconds, microsecond and nanoseconds are represented as fixed floating
-  # point numbers with 3, 6 and 9 decimal places respectively.
-  expect_dplyr_equal(
-    input %>%
-      mutate(x = strftime(datetime, format = "%S")) %>%
-      transmute(as.double(substr(x, 1, 2))) %>%
-      collect(),
-    times,
-    tolerance = 1e-6
-  )
-})
-
-test_that("format_ISO8601", {
-  skip_on_os("windows") # https://issues.apache.org/jira/browse/ARROW-13168
-  times <- tibble(x = c(lubridate::ymd_hms("2018-10-07 19:04:05", tz = "Etc/GMT+6"), NA))
-
-  expect_dplyr_equal(
-    input %>%
-      mutate(x = format_ISO8601(x, precision = "ymd", usetz = FALSE)) %>%
-      collect(),
-    times
-  )
-
-  if (getRversion() < "3.5") {
-    # before 3.5, times$x will have no timezone attribute, so Arrow faithfully
-    # errors that there is no timezone to format:
-    expect_error(
-      times %>%
-        Table$create() %>%
-        mutate(x = format_ISO8601(x, precision = "ymd", usetz = TRUE)) %>%
-        collect(),
-      "Timezone not present, cannot convert to string with timezone: %Y-%m-%d%z"
-    )
-
-    # See comment regarding %S flag in strftime tests
-    expect_error(
-      times %>%
-        Table$create() %>%
-        mutate(x = format_ISO8601(x, precision = "ymdhms", usetz = TRUE)) %>%
-        mutate(x = gsub("\\.0*", "", x)) %>%
-        collect(),
-      "Timezone not present, cannot convert to string with timezone: %Y-%m-%dT%H:%M:%S%z"
-    )
-  } else {
-    expect_dplyr_equal(
-      input %>%
-        mutate(x = format_ISO8601(x, precision = "ymd", usetz = TRUE)) %>%
-        collect(),
-      times
-    )
-
-    # See comment regarding %S flag in strftime tests
-    expect_dplyr_equal(
-      input %>%
-        mutate(x = format_ISO8601(x, precision = "ymdhms", usetz = TRUE)) %>%
-        mutate(x = gsub("\\.0*", "", x)) %>%
-        collect(),
-      times
-    )
-  }
-
-
-  # See comment regarding %S flag in strftime tests
-  expect_dplyr_equal(
-    input %>%
-      mutate(x = format_ISO8601(x, precision = "ymdhms", usetz = FALSE)) %>%
-      mutate(x = gsub("\\.0*", "", x)) %>%
-      collect(),
-    times
-  )
-})
-
 test_that("arrow_find_substring and arrow_find_substring_regex", {
   df <- tibble(x = c("Foo and Bar", "baz and qux and quux"))
 
@@ -945,15 +757,15 @@ test_that("stri_reverse and arrow_ascii_reverse functions", {
 
   df_utf8 <- tibble(x = c("Foo\u00A0\u0061nd\u00A0bar", "\u0062az\u00A0and\u00A0qux\u3000and\u00A0quux"))
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(x = stri_reverse(x)) %>%
       collect(),
     df_utf8
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(x = stri_reverse(x)) %>%
       collect(),
     df_ascii
@@ -980,7 +792,7 @@ test_that("str_like", {
   df <- tibble(x = c("Foo and bar", "baz and qux and quux"))
 
   # TODO: After new version of stringr with str_like has been released, update all
-  # these tests to use expect_dplyr_equal
+  # these tests to use compare_dplyr_binding
 
   # No match - entire string
   expect_equal(
@@ -1029,8 +841,8 @@ test_that("str_like", {
 
   # This will give an error until a new version of stringr with str_like has been released
   skip_if_not(packageVersion("stringr") > "1.4.0")
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(x = str_like(x, "%baz%")) %>%
       collect(),
     df
@@ -1040,36 +852,36 @@ test_that("str_like", {
 test_that("str_pad", {
   df <- tibble(x = c("Foo and bar", "baz and qux and quux"))
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(x = str_pad(x, width = 31)) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(x = str_pad(x, width = 30, side = "right")) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(x = str_pad(x, width = 31, side = "left", pad = "+")) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(x = str_pad(x, width = 10, side = "left", pad = "+")) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(x = str_pad(x, width = 31, side = "both")) %>%
       collect(),
     df
@@ -1079,86 +891,87 @@ test_that("str_pad", {
 test_that("substr", {
   df <- tibble(x = "Apache Arrow")
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(y = substr(x, 1, 6)) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(y = substr(x, 0, 6)) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(y = substr(x, -1, 6)) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(y = substr(x, 6, 1)) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(y = substr(x, -1, -2)) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(y = substr(x, 9, 6)) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(y = substr(x, 1, 6)) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(y = substr(x, 8, 12)) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(y = substr(x, -5, -1)) %>%
       collect(),
     df
   )
 
   expect_error(
-    nse_funcs$substr("Apache Arrow", c(1, 2), 3),
+    call_binding("substr", "Apache Arrow", c(1, 2), 3),
     "`start` must be length 1 - other lengths are not supported in Arrow"
   )
 
   expect_error(
-    nse_funcs$substr("Apache Arrow", 1, c(2, 3)),
+    call_binding("substr", "Apache Arrow", 1, c(2, 3)),
     "`stop` must be length 1 - other lengths are not supported in Arrow"
   )
 })
 
 test_that("substring", {
-  # nse_funcs$substring just calls nse_funcs$substr, tested extensively above
+  # binding for substring just calls call_binding("substr", ...),
+  # tested extensively above
   df <- tibble(x = "Apache Arrow")
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(y = substring(x, 1, 6)) %>%
       collect(),
     df
@@ -1168,171 +981,259 @@ test_that("substring", {
 test_that("str_sub", {
   df <- tibble(x = "Apache Arrow")
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(y = str_sub(x, 1, 6)) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(y = str_sub(x, 0, 6)) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(y = str_sub(x, -1, 6)) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(y = str_sub(x, 6, 1)) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(y = str_sub(x, -1, -2)) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(y = str_sub(x, -1, 3)) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(y = str_sub(x, 9, 6)) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(y = str_sub(x, 1, 6)) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(y = str_sub(x, 8, 12)) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(y = str_sub(x, -5, -1)) %>%
       collect(),
     df
   )
 
   expect_error(
-    nse_funcs$str_sub("Apache Arrow", c(1, 2), 3),
+    call_binding("str_sub", "Apache Arrow", c(1, 2), 3),
     "`start` must be length 1 - other lengths are not supported in Arrow"
   )
 
   expect_error(
-    nse_funcs$str_sub("Apache Arrow", 1, c(2, 3)),
+    call_binding("str_sub", "Apache Arrow", 1, c(2, 3)),
     "`end` must be length 1 - other lengths are not supported in Arrow"
   )
 })
 
 test_that("str_starts, str_ends, startsWith, endsWith", {
-  df <- tibble(x = c("Foo", "bar", "baz", "qux"))
+  df <- tibble(x = c("Foo", "bar", "baz", "qux", NA_character_))
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       filter(str_starts(x, "b.*")) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       filter(str_starts(x, "b.*", negate = TRUE)) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       filter(str_starts(x, fixed("b.*"))) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       filter(str_starts(x, fixed("b"))) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
+      transmute(a = str_starts(x, "b.*"),
+                b = str_starts(x, "b.*", negate = TRUE),
+                c = str_starts(x, fixed("b")),
+                d = str_starts(x, fixed("b"), negate = TRUE)
+                ) %>%
+      collect(),
+    df
+  )
+
+  compare_dplyr_binding(
+    .input %>%
       filter(str_ends(x, "r")) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       filter(str_ends(x, "r", negate = TRUE)) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       filter(str_ends(x, fixed("r$"))) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       filter(str_ends(x, fixed("r"))) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
+      transmute(a = str_ends(x, "r"),
+                b = str_ends(x, "r", negate = TRUE),
+                c = str_ends(x, fixed("r")),
+                d = str_ends(x, fixed("r"), negate = TRUE)) %>%
+      collect(),
+    df
+  )
+  compare_dplyr_binding(
+    .input %>%
       filter(startsWith(x, "b")) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       filter(endsWith(x, "r")) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       filter(startsWith(x, "b.*")) %>%
       collect(),
     df
   )
 
-  expect_dplyr_equal(
-    input %>%
+  compare_dplyr_binding(
+    .input %>%
       filter(endsWith(x, "r$")) %>%
       collect(),
     df
   )
+
+  compare_dplyr_binding(
+    .input %>%
+      transmute(a = startsWith(x, "b"),
+                b = endsWith(x, "r")) %>%
+      collect(),
+    df
+  )
+})
+
+test_that("str_count", {
+  df <- tibble(
+    cities = c("Kolkata", "Dar es Salaam", "Tel Aviv", "San Antonio", "Cluj Napoca", "Bern", "Bogota"),
+    dots = c("a.", "...", ".a.a", "a..a.", "ab...", "dse....", ".f..d..")
+  )
+
+  compare_dplyr_binding(
+    .input %>%
+      mutate(a_count = str_count(cities, pattern = "a")) %>%
+      collect(),
+    df
+  )
+
+  compare_dplyr_binding(
+    .input %>%
+      mutate(p_count = str_count(cities, pattern = "d")) %>%
+      collect(),
+    df
+  )
+
+  compare_dplyr_binding(
+    .input %>%
+      mutate(p_count = str_count(cities,
+        pattern = regex("d", ignore_case = TRUE)
+      )) %>%
+      collect(),
+    df
+  )
+
+  compare_dplyr_binding(
+    .input %>%
+      mutate(e_count = str_count(cities, pattern = "u")) %>%
+      collect(),
+    df
+  )
+
+  # call_binding("str_count", ) is not vectorised over pattern
+  compare_dplyr_binding(
+    .input %>%
+      mutate(let_count = str_count(cities, pattern = c("a", "b", "e", "g", "p", "n", "s"))) %>%
+      collect(),
+    df,
+    warning = TRUE
+  )
+
+  compare_dplyr_binding(
+    .input %>%
+      mutate(dots_count = str_count(dots, ".")) %>%
+      collect(),
+    df
+  )
+
+  compare_dplyr_binding(
+    .input %>%
+      mutate(dots_count = str_count(dots, fixed("."))) %>%
+      collect(),
+    df
+  )
 })
