@@ -23,6 +23,7 @@
 #include "arrow/buffer.h"
 #include "arrow/filesystem/gcsfs_internal.h"
 #include "arrow/filesystem/path_util.h"
+#include "arrow/filesystem/util_internal.h"
 #include "arrow/result.h"
 #include "arrow/util/checked_cast.h"
 #include "arrow/util/thread_pool.h"
@@ -43,6 +44,7 @@ namespace {
 
 namespace gcs = google::cloud::storage;
 using GcsCode = google::cloud::StatusCode;
+using GcsStatus = google::cloud::Status;
 
 // Change the default upload buffer size. In general, sending larger buffers is more
 // efficient with GCS, as each buffer requires a roundtrip to the service. With formatted
@@ -92,12 +94,11 @@ struct GcsPath {
 
 class GcsInputStream : public arrow::io::InputStream {
  public:
-  explicit GcsInputStream(gcs::ObjectReadStream stream, std::string bucket_name,
-                          std::string object_name, gcs::Generation generation,
-                          gcs::ReadFromOffset offset, gcs::Client client)
+  explicit GcsInputStream(gcs::ObjectReadStream stream, GcsPath path,
+                          gcs::Generation generation, gcs::ReadFromOffset offset,
+                          gcs::Client client)
       : stream_(std::move(stream)),
-        bucket_name_(std::move(bucket_name)),
-        object_name_(std::move(object_name)),
+        path_(std::move(path)),
         generation_(generation),
         offset_(offset.value_or(0)),
         client_(std::move(client)) {}
@@ -138,14 +139,14 @@ class GcsInputStream : public arrow::io::InputStream {
     stream_.read(reinterpret_cast<char*>(buffer->mutable_data()), nbytes);
     ARROW_GCS_RETURN_NOT_OK(stream_.status());
     RETURN_NOT_OK(buffer->Resize(stream_.gcount(), true));
-    return buffer;
+    return std::shared_ptr<Buffer>(std::move(buffer));
   }
   //@}
 
   //@{
   // @name InputStream
   Result<std::shared_ptr<const KeyValueMetadata>> ReadMetadata() override {
-    auto metadata = client_.GetObjectMetadata(bucket_name_, object_name_, generation_);
+    auto metadata = client_.GetObjectMetadata(path_.bucket, path_.object, generation_);
     ARROW_GCS_RETURN_NOT_OK(metadata.status());
     return internal::FromObjectMetadata(*metadata);
   }
@@ -153,8 +154,7 @@ class GcsInputStream : public arrow::io::InputStream {
 
  private:
   mutable gcs::ObjectReadStream stream_;
-  std::string bucket_name_;
-  std::string object_name_;
+  GcsPath path_;
   gcs::Generation generation_;
   std::int64_t offset_;
   gcs::Client client_;
@@ -210,7 +210,7 @@ class GcsOutputStream : public arrow::io::OutputStream {
 };
 
 using InputStreamFactory = std::function<Result<std::shared_ptr<io::InputStream>>(
-    const std::string&, const std::string&, gcs::Generation, gcs::ReadFromOffset)>;
+    gcs::Generation, gcs::ReadFromOffset)>;
 
 class GcsRandomAccessFile : public arrow::io::RandomAccessFile {
  public:
@@ -255,16 +255,14 @@ class GcsRandomAccessFile : public arrow::io::RandomAccessFile {
   Result<int64_t> ReadAt(int64_t position, int64_t nbytes, void* out) override {
     if (closed()) return Status::Invalid("Cannot read from closed file");
     std::shared_ptr<io::InputStream> stream;
-    ARROW_ASSIGN_OR_RAISE(stream, factory_(metadata_.bucket(), metadata_.name(),
-                                           gcs::Generation(metadata_.generation()),
+    ARROW_ASSIGN_OR_RAISE(stream, factory_(gcs::Generation(metadata_.generation()),
                                            gcs::ReadFromOffset(position)));
     return stream->Read(nbytes, out);
   }
   Result<std::shared_ptr<Buffer>> ReadAt(int64_t position, int64_t nbytes) override {
     if (closed()) return Status::Invalid("Cannot read from closed file");
     std::shared_ptr<io::InputStream> stream;
-    ARROW_ASSIGN_OR_RAISE(stream, factory_(metadata_.bucket(), metadata_.name(),
-                                           gcs::Generation(metadata_.generation()),
+    ARROW_ASSIGN_OR_RAISE(stream, factory_(gcs::Generation(metadata_.generation()),
                                            gcs::ReadFromOffset(position)));
     return stream->Read(nbytes);
   }
@@ -273,8 +271,7 @@ class GcsRandomAccessFile : public arrow::io::RandomAccessFile {
   // from Seekable
   Status Seek(int64_t position) override {
     if (closed()) return Status::Invalid("Cannot seek in a closed file");
-    ARROW_ASSIGN_OR_RAISE(stream_, factory_(metadata_.bucket(), metadata_.name(),
-                                            gcs::Generation(metadata_.generation()),
+    ARROW_ASSIGN_OR_RAISE(stream_, factory_(gcs::Generation(metadata_.generation()),
                                             gcs::ReadFromOffset(position)));
     return Status::OK();
   }
@@ -410,7 +407,8 @@ class GcsFileSystem::Impl {
       google::cloud::StatusOr<gcs::BucketMetadata> b = client_.GetBucketMetadata(bucket);
       if (!b) {
         if (b.status().code() == GcsCode::kNotFound) {
-          b = client_.CreateBucket(bucket, gcs::BucketMetadata());
+          b = client_.CreateBucket(bucket, gcs::BucketMetadata().set_location(
+                                               options_.default_bucket_location));
         }
         if (!b) return internal::ToArrowStatus(b.status());
       }
@@ -433,7 +431,10 @@ class GcsFileSystem::Impl {
   Status CreateDir(const GcsPath& p) {
     if (p.object.empty()) {
       return internal::ToArrowStatus(
-          client_.CreateBucket(p.bucket, gcs::BucketMetadata()).status());
+          client_
+              .CreateBucket(p.bucket, gcs::BucketMetadata().set_location(
+                                          options_.default_bucket_location))
+              .status());
     }
     auto parent = p.parent();
     if (!parent.object.empty()) {
@@ -448,14 +449,15 @@ class GcsFileSystem::Impl {
   }
 
   Status DeleteDir(const GcsPath& p, const io::IOContext& io_context) {
-    RETURN_NOT_OK(DeleteDirContents(p, io_context));
+    RETURN_NOT_OK(DeleteDirContents(p, /*missing_dir_ok=*/false, io_context));
     if (!p.object.empty()) {
       return internal::ToArrowStatus(client_.DeleteObject(p.bucket, p.object));
     }
     return internal::ToArrowStatus(client_.DeleteBucket(p.bucket));
   }
 
-  Status DeleteDirContents(const GcsPath& p, const io::IOContext& io_context) {
+  Status DeleteDirContents(const GcsPath& p, bool missing_dir_ok,
+                           const io::IOContext& io_context) {
     // If the directory marker exists, it better be a directory.
     auto dir = client_.GetObjectMetadata(p.bucket, p.object);
     if (dir && !IsDirectory(*dir)) return NotDirectoryError(*dir);
@@ -476,10 +478,17 @@ class GcsFileSystem::Impl {
     std::vector<Future<>> submitted;
     // This iterates over all the objects, and schedules parallel deletes.
     auto prefix = p.object.empty() ? gcs::Prefix() : gcs::Prefix(canonical);
+    bool at_least_one_obj = false;
     for (const auto& o : client_.ListObjects(p.bucket, prefix)) {
+      at_least_one_obj = true;
       submitted.push_back(DeferNotOk(io_context.executor()->Submit(async_delete, o)));
     }
 
+    if (!missing_dir_ok && !at_least_one_obj && !dir) {
+      // No files were found and no directory marker exists
+      return Status::IOError("No such directory: ", p.full_path);
+    }
+
     return AllFinished(submitted).status();
   }
 
@@ -530,26 +539,30 @@ class GcsFileSystem::Impl {
     return internal::ToArrowStatus(metadata.status());
   }
 
-  Result<std::shared_ptr<io::InputStream>> OpenInputStream(const std::string& bucket_name,
-                                                           const std::string& object_name,
+  Result<std::shared_ptr<io::InputStream>> OpenInputStream(const GcsPath& path,
                                                            gcs::Generation generation,
                                                            gcs::ReadFromOffset offset) {
-    auto stream = client_.ReadObject(bucket_name, object_name, generation, offset);
+    auto stream = client_.ReadObject(path.bucket, path.object, generation, offset);
     ARROW_GCS_RETURN_NOT_OK(stream.status());
-    return std::make_shared<GcsInputStream>(std::move(stream), bucket_name, object_name,
-                                            gcs::Generation(), offset, client_);
+    return std::make_shared<GcsInputStream>(std::move(stream), path, gcs::Generation(),
+                                            offset, client_);
   }
 
   Result<std::shared_ptr<io::OutputStream>> OpenOutputStream(
       const GcsPath& path, const std::shared_ptr<const KeyValueMetadata>& metadata) {
+    std::shared_ptr<const KeyValueMetadata> resolved_metadata = metadata;
+    if (resolved_metadata == nullptr && options_.default_metadata != nullptr) {
+      resolved_metadata = options_.default_metadata;
+    }
     gcs::EncryptionKey encryption_key;
-    ARROW_ASSIGN_OR_RAISE(encryption_key, internal::ToEncryptionKey(metadata));
+    ARROW_ASSIGN_OR_RAISE(encryption_key, internal::ToEncryptionKey(resolved_metadata));
     gcs::PredefinedAcl predefined_acl;
-    ARROW_ASSIGN_OR_RAISE(predefined_acl, internal::ToPredefinedAcl(metadata));
+    ARROW_ASSIGN_OR_RAISE(predefined_acl, internal::ToPredefinedAcl(resolved_metadata));
     gcs::KmsKeyName kms_key_name;
-    ARROW_ASSIGN_OR_RAISE(kms_key_name, internal::ToKmsKeyName(metadata));
+    ARROW_ASSIGN_OR_RAISE(kms_key_name, internal::ToKmsKeyName(resolved_metadata));
     gcs::WithObjectMetadata with_object_metadata;
-    ARROW_ASSIGN_OR_RAISE(with_object_metadata, internal::ToObjectMetadata(metadata));
+    ARROW_ASSIGN_OR_RAISE(with_object_metadata,
+                          internal::ToObjectMetadata(resolved_metadata));
 
     auto stream = client_.WriteObject(path.bucket, path.object, encryption_key,
                                       predefined_acl, kms_key_name, with_object_metadata);
@@ -610,46 +623,106 @@ class GcsFileSystem::Impl {
 
 bool GcsOptions::Equals(const GcsOptions& other) const {
   return credentials == other.credentials &&
-         endpoint_override == other.endpoint_override && scheme == other.scheme;
+         endpoint_override == other.endpoint_override && scheme == other.scheme &&
+         default_bucket_location == other.default_bucket_location;
 }
 
 GcsOptions GcsOptions::Defaults() {
-  return GcsOptions{
-      std::make_shared<GcsCredentials>(google::cloud::MakeGoogleDefaultCredentials()),
-      {},
-      "https"};
+  GcsOptions options{};
+  options.credentials =
+      std::make_shared<GcsCredentials>(google::cloud::MakeGoogleDefaultCredentials());
+  options.scheme = "https";
+  return options;
 }
 
 GcsOptions GcsOptions::Anonymous() {
-  return GcsOptions{
-      std::make_shared<GcsCredentials>(google::cloud::MakeInsecureCredentials()),
-      {},
-      "http"};
+  GcsOptions options{};
+  options.credentials =
+      std::make_shared<GcsCredentials>(google::cloud::MakeInsecureCredentials());
+  options.scheme = "http";
+  return options;
 }
 
 GcsOptions GcsOptions::FromAccessToken(const std::string& access_token,
                                        std::chrono::system_clock::time_point expiration) {
-  return GcsOptions{
-      std::make_shared<GcsCredentials>(
-          google::cloud::MakeAccessTokenCredentials(access_token, expiration)),
-      {},
-      "https"};
+  GcsOptions options{};
+  options.credentials = std::make_shared<GcsCredentials>(
+      google::cloud::MakeAccessTokenCredentials(access_token, expiration));
+  options.scheme = "https";
+  return options;
 }
 
 GcsOptions GcsOptions::FromImpersonatedServiceAccount(
     const GcsCredentials& base_credentials, const std::string& target_service_account) {
-  return GcsOptions{std::make_shared<GcsCredentials>(
-                        google::cloud::MakeImpersonateServiceAccountCredentials(
-                            base_credentials.credentials, target_service_account)),
-                    {},
-                    "https"};
+  GcsOptions options{};
+  options.credentials = std::make_shared<GcsCredentials>(
+      google::cloud::MakeImpersonateServiceAccountCredentials(
+          base_credentials.credentials, target_service_account));
+  options.scheme = "https";
+  return options;
 }
 
 GcsOptions GcsOptions::FromServiceAccountCredentials(const std::string& json_object) {
-  return GcsOptions{std::make_shared<GcsCredentials>(
-                        google::cloud::MakeServiceAccountCredentials(json_object)),
-                    {},
-                    "https"};
+  GcsOptions options{};
+  options.credentials = std::make_shared<GcsCredentials>(
+      google::cloud::MakeServiceAccountCredentials(json_object));
+  options.scheme = "https";
+  return options;
+}
+
+Result<GcsOptions> GcsOptions::FromUri(const arrow::internal::Uri& uri,
+                                       std::string* out_path) {
+  const auto bucket = uri.host();
+  auto path = uri.path();
+  if (bucket.empty()) {
+    if (!path.empty()) {
+      return Status::Invalid("Missing bucket name in GCS URI");
+    }
+  } else {
+    if (path.empty()) {
+      path = bucket;
+    } else {
+      if (path[0] != '/') {
+        return Status::Invalid("GCS URI should be absolute, not relative");
+      }
+      path = bucket + path;
+    }
+  }
+  if (out_path != nullptr) {
+    *out_path = std::string(internal::RemoveTrailingSlash(path));
+  }
+
+  std::unordered_map<std::string, std::string> options_map;
+  ARROW_ASSIGN_OR_RAISE(const auto options_items, uri.query_items());
+  for (const auto& kv : options_items) {
+    options_map.emplace(kv.first, kv.second);
+  }
+
+  if (!uri.password().empty() || !uri.username().empty()) {
+    return Status::Invalid("GCS does not accept username or password.");
+  }
+
+  auto options = GcsOptions::Defaults();
+  for (const auto& kv : options_map) {
+    if (kv.first == "location") {
+      options.default_bucket_location = kv.second;
+    } else if (kv.first == "scheme") {
+      options.scheme = kv.second;
+    } else if (kv.first == "endpoint_override") {
+      options.endpoint_override = kv.second;
+    } else {
+      return Status::Invalid("Unexpected query parameter in GCS URI: '", kv.first, "'");
+    }
+  }
+
+  return options;
+}
+
+Result<GcsOptions> GcsOptions::FromUri(const std::string& uri_string,
+                                       std::string* out_path) {
+  arrow::internal::Uri uri;
+  RETURN_NOT_OK(uri.Parse(uri_string));
+  return FromUri(uri, out_path);
 }
 
 std::string GcsFileSystem::type_name() const { return "gcs"; }
@@ -685,9 +758,9 @@ Status GcsFileSystem::DeleteDir(const std::string& path) {
   return impl_->DeleteDir(p, io_context());
 }
 
-Status GcsFileSystem::DeleteDirContents(const std::string& path) {
+Status GcsFileSystem::DeleteDirContents(const std::string& path, bool missing_dir_ok) {
   ARROW_ASSIGN_OR_RAISE(auto p, GcsPath::FromString(path));
-  return impl_->DeleteDirContents(p, io_context());
+  return impl_->DeleteDirContents(p, missing_dir_ok, io_context());
 }
 
 Status GcsFileSystem::DeleteRootDirContents() {
@@ -716,8 +789,7 @@ Status GcsFileSystem::CopyFile(const std::string& src, const std::string& dest)
 Result<std::shared_ptr<io::InputStream>> GcsFileSystem::OpenInputStream(
     const std::string& path) {
   ARROW_ASSIGN_OR_RAISE(auto p, GcsPath::FromString(path));
-  return impl_->OpenInputStream(p.bucket, p.object, gcs::Generation(),
-                                gcs::ReadFromOffset());
+  return impl_->OpenInputStream(p, gcs::Generation(), gcs::ReadFromOffset());
 }
 
 Result<std::shared_ptr<io::InputStream>> GcsFileSystem::OpenInputStream(
@@ -727,8 +799,7 @@ Result<std::shared_ptr<io::InputStream>> GcsFileSystem::OpenInputStream(
                            "' as an input stream");
   }
   ARROW_ASSIGN_OR_RAISE(auto p, GcsPath::FromString(info.path()));
-  return impl_->OpenInputStream(p.bucket, p.object, gcs::Generation(),
-                                gcs::ReadFromOffset());
+  return impl_->OpenInputStream(p, gcs::Generation(), gcs::ReadFromOffset());
 }
 
 Result<std::shared_ptr<io::RandomAccessFile>> GcsFileSystem::OpenInputFile(
@@ -737,14 +808,12 @@ Result<std::shared_ptr<io::RandomAccessFile>> GcsFileSystem::OpenInputFile(
   auto metadata = impl_->GetObjectMetadata(p);
   ARROW_GCS_RETURN_NOT_OK(metadata.status());
   auto impl = impl_;
-  auto open_stream = [impl](const std::string& b, const std::string& o, gcs::Generation g,
-                            gcs::ReadFromOffset offset) {
-    return impl->OpenInputStream(b, o, g, offset);
+  auto open_stream = [impl, p](gcs::Generation g, gcs::ReadFromOffset offset) {
+    return impl->OpenInputStream(p, g, offset);
   };
-  ARROW_ASSIGN_OR_RAISE(
-      auto stream,
-      impl_->OpenInputStream(p.bucket, p.object, gcs::Generation(metadata->generation()),
-                             gcs::ReadFromOffset()));
+  ARROW_ASSIGN_OR_RAISE(auto stream,
+                        impl_->OpenInputStream(p, gcs::Generation(metadata->generation()),
+                                               gcs::ReadFromOffset()));
 
   return std::make_shared<GcsRandomAccessFile>(std::move(open_stream),
                                                *std::move(metadata), std::move(stream));
@@ -760,14 +829,12 @@ Result<std::shared_ptr<io::RandomAccessFile>> GcsFileSystem::OpenInputFile(
   auto metadata = impl_->GetObjectMetadata(p);
   ARROW_GCS_RETURN_NOT_OK(metadata.status());
   auto impl = impl_;
-  auto open_stream = [impl](const std::string& b, const std::string& o, gcs::Generation g,
-                            gcs::ReadFromOffset offset) {
-    return impl->OpenInputStream(b, o, g, offset);
+  auto open_stream = [impl, p](gcs::Generation g, gcs::ReadFromOffset offset) {
+    return impl->OpenInputStream(p, g, offset);
   };
-  ARROW_ASSIGN_OR_RAISE(
-      auto stream,
-      impl_->OpenInputStream(p.bucket, p.object, gcs::Generation(metadata->generation()),
-                             gcs::ReadFromOffset()));
+  ARROW_ASSIGN_OR_RAISE(auto stream,
+                        impl_->OpenInputStream(p, gcs::Generation(metadata->generation()),
+                                               gcs::ReadFromOffset()));
 
   return std::make_shared<GcsRandomAccessFile>(std::move(open_stream),
                                                *std::move(metadata), std::move(stream));
@@ -787,8 +854,7 @@ Result<std::shared_ptr<io::OutputStream>> GcsFileSystem::OpenAppendStream(
 std::shared_ptr<GcsFileSystem> GcsFileSystem::Make(const GcsOptions& options,
                                                    const io::IOContext& context) {
   // Cannot use `std::make_shared<>` as the constructor is private.
-  return std::shared_ptr<GcsFileSystem>(
-      new GcsFileSystem(options, io::default_io_context()));
+  return std::shared_ptr<GcsFileSystem>(new GcsFileSystem(options, context));
 }
 
 GcsFileSystem::GcsFileSystem(const GcsOptions& options, const io::IOContext& context)
